var CHESSBOARD = {
    canSelect: true,
    canSound: true,
    playersnames: {
        blackPlayer: '',
        whitePlayer: ''
    },
    chessBoardColumnsBlack: {
        1: 'h',
        2: 'g',
        3: 'f',
        4: 'e',
        5: 'd',
        6: 'c',
        7: 'b',
        8: 'a'
    },
    chessBoardColumnsWhite: {
        1: 'a',
        2: 'b',
        3: 'c',
        4: 'd',
        5: 'e',
        6: 'f',
        7: 'g',
        8: 'h'
    },
    chessBoardColumns: {
        1: 'a',
        2: 'b',
        3: 'c',
        4: 'd',
        5: 'e',
        6: 'f',
        7: 'g',
        8: 'h'
    },
    chessBoardRowsBlack: [1, 2, 3, 4, 5, 6, 7, 8],
    chessBoardRowsWhite: [8, 7, 6, 5, 4, 3, 2, 1],
    chessBoardRows: [8, 7, 6, 5, 4, 3, 2, 1],
    colors: {
        'w': 'white',
        'b': 'black'
    },
    //config <
    checkedWin: false,
    firstmove: {
        white: {
            king: true,
            queen: true
        },
        black: {
            king: true,
            queen: true
        }
    },
    pieces: [],
    gameId: '',
    gameTimer: 0, // Timer set for the current game
    selectedPiece: '', // selector for the piece held by the user
    playerA: '', // playerA username
    playerB: '', // playerB username
    blackPlayer: '', // blackPlayer username
    blackTimer: 0, // blackPlayer timer, in seconds
    lastcout: {
        'w': -1,
        'b': -1
    },
    whitePlayer: '', // whitePlayer username
    whiteTimer: 0, // whitePlayer timer, in seconds
    timer: '', // set the current value in seconds of the timer being counted down (white or black)
    timerInterval: '', // set to an interval when a countdown is required
    currentGameTurn: '', // 'w' or 'b'
    gameHistory: [], // turns history
    //config >

    chessBoard: {
        11: '',
        12: '',
        13: '',
        14: '',
        15: '',
        16: '',
        17: '',
        18: '',
        21: '',
        22: '',
        23: '',
        24: '',
        25: '',
        26: '',
        27: '',
        28: '',
        31: '',
        32: '',
        33: '',
        34: '',
        35: '',
        36: '',
        37: '',
        38: '',
        41: '',
        42: '',
        43: '',
        44: '',
        45: '',
        46: '',
        47: '',
        48: '',
        51: '',
        52: '',
        53: '',
        54: '',
        55: '',
        56: '',
        57: '',
        58: '',
        61: '',
        62: '',
        63: '',
        64: '',
        65: '',
        66: '',
        67: '',
        68: '',
        71: '',
        72: '',
        73: '',
        74: '',
        75: '',
        76: '',
        77: '',
        78: '',
        81: '',
        82: '',
        83: '',
        84: '',
        85: '',
        86: '',
        87: '',
        88: ''
    },

    numericColumns: {
        'a': 10,
        'b': 20,
        'c': 30,
        'd': 40,
        'e': 50,
        'f': 60,
        'g': 70,
        'h': 80
    },
    unnumericColumns: {
        1: "a",
        2: "b",
        3: "c",
        4: "d",
        5: "e",
        6: "f",
        7: "g",
        8: "h"
    },
    ///////////////////////////////
    // UTILITIES
    ///////////////////////////////
    _attack_by_king: function (pieceId, destinationId) {
        var from = CHESSBOARD.pieces[pieceId].sqId;
        var numericFrom = parseInt(CHESSBOARD._numeric(from), 10);
        var numericTo = parseInt(CHESSBOARD._numeric(destinationId), 10);
        var numericMove = numericTo - numericFrom;
        if (numericMove === 1 || numericMove === -1 || numericMove === -11 || numericMove === 11 || numericMove === -9 ||
            numericMove === 9 || numericMove === -10 || numericMove === 10) {
            return true;
        } else {
            return false;
        }
    },
    generateMovePositions: function (x) {
        forReturn = [];
        try {
        for (var i = 1; i <= 8; i++) {
            for (var j = 1; j <= 8; j++) {
                numeric = i * 10 + j;
                sqid = CHESSBOARD._unnumeric(numeric);
                if (CHESSBOARD._canMove(CHESSBOARD.pieces[x].id, sqid)) {
                    if (CHESSBOARD.chessBoard[numeric] != '' && CHESSBOARD.pieces[x].id.charAt(0) == CHESSBOARD.pieces[CHESSBOARD.chessBoard[numeric]].id.charAt(0))
                        continue;
                    if (!CHESSBOARD._isCheck(CHESSBOARD.pieces[x].id, sqid))
                        forReturn.push(sqid);
                }
            }
        }
        } catch (e) {} finally {return forReturn;}
        return forReturn;
    },
    _canMove: function (pieceId, destinationId, captureOnly) {
        var operator = 0;
        var i = "";
        var targetPiece = CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId)];
        var from = CHESSBOARD.pieces[pieceId].sqId;
        var numericFrom = parseInt(CHESSBOARD._numeric(from), 10);
        var numericTo = parseInt(CHESSBOARD._numeric(destinationId), 10);
        var numericMove = numericTo - numericFrom;
        //raksa move rule
        if (targetPiece) {
            return false;
        }
        if (from[2] === destinationId[2]
            || from[3] === destinationId[3]) {
            if (from[2] === destinationId[2] && numericMove > 0) {
                operator = 1;
            } else if (from[2] === destinationId[2] && numericMove < 0) {
                operator = -1;
            } else if (from[3] === destinationId[3] && numericMove > 0) {
                operator = 10;
            } else if (from[3] === destinationId[3] && numericMove < 0) {
                operator = -10;
            }
            for (i = numericFrom + operator; (i < numericTo && numericMove > 0) || (i > numericTo && numericMove < 0); i += operator) {
                if (CHESSBOARD.chessBoard[i]) {
                    return false;
                }
            }
            return true;
        }
        return false;
    },
    explorSeround: function (sqId, color) {
        if (CHESSBOARD.chessBoard[CHESSBOARD._numeric(sqId)] === '')
            return false;
        if (CHESSBOARD.chessBoard[CHESSBOARD._numeric(sqId)][0] !== color)
            return false;
        var bool = true;
        for (var x in CHESSBOARD.arrseround) {
            if (CHESSBOARD.arrseround[x] === CHESSBOARD.chessBoard[CHESSBOARD._numeric(sqId)])
                bool = false;
        }
        if (!bool)
            return false;
        CHESSBOARD.arrseround.push(CHESSBOARD.chessBoard[CHESSBOARD._numeric(sqId)]);
        var num_id = CHESSBOARD._numeric(sqId);
        if (CHESSBOARD._numeric(sqId) % 10 >= 2)
            CHESSBOARD.explorSeround(CHESSBOARD._unnumeric(num_id - 1), color);
        if (CHESSBOARD._numeric(sqId) % 10 <= 7)
            CHESSBOARD.explorSeround(CHESSBOARD._unnumeric(num_id + 1), color);
        if (Math.floor(CHESSBOARD._numeric(sqId) / 10) >= 2)
            CHESSBOARD.explorSeround(CHESSBOARD._unnumeric(num_id - 10), color);
        if (Math.floor(CHESSBOARD._numeric(sqId) / 10) <= 7)
            CHESSBOARD.explorSeround(CHESSBOARD._unnumeric(num_id + 10), color);
    },
    checkSerounded: function (piece) {
        var bool = true;
        CHESSBOARD.arrseround = [];
        CHESSBOARD.explorSeround(CHESSBOARD.pieces[piece].sqId, piece[0]);
        var arr_pieces = CHESSBOARD.arrseround;
        for (var indp in arr_pieces) {
            var sqId = CHESSBOARD.pieces[arr_pieces[indp]].sqId;
            var num_id = CHESSBOARD._numeric(sqId);
            var arr_num_id = [];
            if (CHESSBOARD._numeric(sqId) % 10 >= 2)
                arr_num_id.push(num_id - 1);
            if (CHESSBOARD._numeric(sqId) % 10 <= 7)
                arr_num_id.push(num_id + 1);
            if (Math.floor(CHESSBOARD._numeric(sqId) / 10) >= 2)
                arr_num_id.push(num_id - 10);
            if (Math.floor(CHESSBOARD._numeric(sqId) / 10) <= 7)
                arr_num_id.push(num_id + 10);
            for (var x in arr_num_id) {
                if (CHESSBOARD._canMove(arr_pieces[indp], CHESSBOARD._unnumeric(arr_num_id[x])))
                    bool = false;
            }
        }
        if (bool) {
            for (var indp in arr_pieces) {
                CHESSBOARD.move(arr_pieces[indp], CHESSBOARD.pieces[arr_pieces[indp]].id[0] + 'Graveyard', true);
            }
            for (var indp in arr_pieces) {
                if (CHESSBOARD.pieces[arr_pieces[indp]].type === 'king') {
                    CHESSBOARD.checkedWin = true;
                    CHESSBOARD.stopTimer();
                    oukonline.boardWaiting(true, (piece[0] === 'b' ? 'White' : 'Black') + ' wins.', true, 'opacity', 0.7);
                }
            }
        }
    },
    killSerounded: function (sqId, color) {
        var num_id = CHESSBOARD._numeric(sqId);
        var arr_num_id = [];
        if (CHESSBOARD._numeric(sqId) % 10 >= 2)
            arr_num_id.push(num_id - 1);
        if (CHESSBOARD._numeric(sqId) % 10 <= 7)
            arr_num_id.push(num_id + 1);
        if (Math.floor(CHESSBOARD._numeric(sqId) / 10) >= 2)
            arr_num_id.push(num_id - 10);
        if (Math.floor(CHESSBOARD._numeric(sqId) / 10) <= 7)
            arr_num_id.push(num_id + 10);
        for (var x in arr_num_id) {
            if (CHESSBOARD.chessBoard[arr_num_id[x]] !== '' && CHESSBOARD.chessBoard[arr_num_id[x]][0] !== color) {
                CHESSBOARD.checkSerounded(CHESSBOARD.chessBoard[arr_num_id[x]]);
            }
        }
    },
    _CommitMoves: function (msg) { //raksa
        ////console.log(msg);
        oukonline.Gameactivity(CHESSBOARD.gameId, msg);
    },
    _createPieces: function () { //*****
        for (var c in CHESSBOARD.colors) {
            CHESSBOARD.pieces[c + 'king'] = {
                id: c + 'king',
                type: 'king',
                order: 0,
                name: CHESSBOARD.colors[c] + ' king',
                classname: c + 'king',
                sqId: 'sq' + (c === "w" ? 'h' : 'a') + (c === "w" ? 2 : 7),
                initx: (c === "w" ? 8 : 1),
                inity: (c === "w" ? 2 : 7),
                isCheck: false
            };
            CHESSBOARD.pieces[c + 'queen'] = {
                id: c + 'queen',
                type: 'queen',
                order: 1,
                name: CHESSBOARD.colors[c] + ' queen',
                classname: c + 'queen',
                sqId: 'sq' + (c === "w" ? 'e' : 'd') + (c === "w" ? 1 : 8),
                initx: 4,
                inity: (c === "w" ? 1 : 8)
            };
            CHESSBOARD.pieces[c + 'rooka'] = {
                id: c + 'rooka',
                type: 'rook',
                order: 2,
                name: CHESSBOARD.colors[c] + ' rook (a)',
                classname: c + 'rook',
                sqId: 'sq' + (c === "w" ? 'a' : 'e') + (c === "w" ? 1 : 8),
                initx: (c === "w" ? 1 : 5),
                inity: (c === "w" ? 1 : 8)
            };
            CHESSBOARD.pieces[c + 'rookh'] = {
                id: c + 'rookh',
                type: 'rook',
                order: 3,
                name: CHESSBOARD.colors[c] + ' rook (h)',
                classname: c + 'rook',
                sqId: 'sq' + (c === "w" ? 'd' : 'h') + (c === "w" ? 1 : 8),
                initx: (c === "w" ? 4 : 8),
                inity: (c === "w" ? 1 : 8)
            };
            CHESSBOARD.pieces[c + 'knightb'] = {
                id: c + 'knightb',
                type: 'knight',
                order: 4,
                name: CHESSBOARD.colors[c] + ' knight (b)',
                classname: c + 'knight',
                sqId: 'sq' + 'b' + (c === "w" ? 1 : 8),
                initx: 2,
                inity: (c === "w" ? 1 : 8)
            };
            CHESSBOARD.pieces[c + 'knightg'] = {
                id: c + 'knightg',
                type: 'knight',
                order: 2,
                name: CHESSBOARD.colors[c] + ' knight (g)',
                classname: c + 'knight',
                sqId: 'sq' + 'g' + (c === "w" ? 1 : 8),
                initx: 7,
                inity: (c === "w" ? 1 : 8)
            };
            CHESSBOARD.pieces[c + 'bishopc'] = {
                id: c + 'bishopc',
                type: 'bishop',
                order: 6,
                name: CHESSBOARD.colors[c] + ' bishop (c)',
                classname: c + 'bishop',
                sqId: 'sq' + 'c' + (c === "w" ? 1 : 8),
                initx: 3,
                inity: (c === "w" ? 1 : 8)
            };
            CHESSBOARD.pieces[c + 'bishopf'] = {
                id: c + 'bishopf',
                type: 'bishop',
                order: 7,
                name: CHESSBOARD.colors[c] + ' bishop (f)',
                classname: c + 'bishop',
                sqId: 'sq' + 'f' + (c === "w" ? 1 : 8),
                initx: 6,
                inity: (c === "w" ? 1 : 8)
            };
            for (var i in CHESSBOARD.chessBoardColumns) {
                CHESSBOARD.pieces[c + 'pawn' + CHESSBOARD.chessBoardColumns[i]] = {
                    id: c + 'pawn' + CHESSBOARD.chessBoardColumns[i],
                    type: 'pawn',
                    order: 7 + i,
                    name: CHESSBOARD.colors[c] + ' pawn (' + CHESSBOARD.chessBoardColumns[i] + ')',
                    classname: c + 'pawn',
                    sqId: 'sq' + CHESSBOARD.chessBoardColumns[i] + (c === "w" ? 3 : 6),
                    initx: i,
                    inity: (c === "w" ? 3 : 6)
                };
            }
        }
        // init chessBoard array
        CHESSBOARD.chessBoard = {
            11: '',
            12: '',
            13: '',
            14: '',
            15: '',
            16: '',
            17: '',
            18: '',
            21: '',
            22: '',
            23: '',
            24: '',
            25: '',
            26: '',
            27: '',
            28: '',
            31: '',
            32: '',
            33: '',
            34: '',
            35: '',
            36: '',
            37: '',
            38: '',
            41: '',
            42: '',
            43: '',
            44: '',
            45: '',
            46: '',
            47: '',
            48: '',
            51: '',
            52: '',
            53: '',
            54: '',
            55: '',
            56: '',
            57: '',
            58: '',
            61: '',
            62: '',
            63: '',
            64: '',
            65: '',
            66: '',
            67: '',
            68: '',
            71: '',
            72: '',
            73: '',
            74: '',
            75: '',
            76: '',
            77: '',
            78: '',
            81: '',
            82: '',
            83: '',
            84: '',
            85: '',
            86: '',
            87: '',
            88: ''
        };
        for (var piece in CHESSBOARD.pieces) {
            CHESSBOARD.chessBoard[CHESSBOARD._numeric(CHESSBOARD.pieces[piece].sqId)] = CHESSBOARD.pieces[piece].id;
        }
    },
    _drawBoard: function () {
        var board = $('#chessBoard');
        for (var row in CHESSBOARD.chessBoardRows) {
            var htmlRow = '<div class="chessBoardRow">';
            for (var column in CHESSBOARD.chessBoardColumns) {
                htmlRow += '<div id="sq' + CHESSBOARD.chessBoardColumns[column] + CHESSBOARD.chessBoardRows[row] + '" class="chessBoardSquare chessBoardSquare' + ((parseInt(column, 10) + parseInt(row, 10)) % 2) + '"></div>';
            }
            htmlRow += '</div>';
            board.append(htmlRow);
        }
        $(".chessBoardSquare").bind('vmousedown', function (event) {
            CHESSBOARD.mouseDownHandler(event);
        }); // JQUERY
    },
    flip: function (bottomColor) {
        if (bottomColor === 'w') {
            CHESSBOARD.chessBoardColumns = CHESSBOARD.chessBoardColumnsWhite;
            CHESSBOARD.chessBoardRows = CHESSBOARD.chessBoardRowsWhite;
        } else if (bottomColor === 'b') {
            CHESSBOARD.chessBoardColumns = CHESSBOARD.chessBoardColumnsBlack;
            CHESSBOARD.chessBoardRows = CHESSBOARD.chessBoardRowsBlack;
        } else { // no bottom color or an illegal bottom color was provided
            // inverses the board without reinitializing it
            var tmpchessBoardColumns = {
                1: CHESSBOARD.chessBoardColumns[8],
                2: CHESSBOARD.chessBoardColumns[7],
                3: CHESSBOARD.chessBoardColumns[6],
                4: CHESSBOARD.chessBoardColumns[5],
                5: CHESSBOARD.chessBoardColumns[4],
                6: CHESSBOARD.chessBoardColumns[3],
                7: CHESSBOARD.chessBoardColumns[2],
                8: CHESSBOARD.chessBoardColumns[1]
            };
            CHESSBOARD.chessBoardColumns = tmpchessBoardColumns;
            CHESSBOARD.chessBoardRows.reverse();
        }
        // JQUERY
        $('#chessBoard').empty(); // remove all children (rows & colums & pieces)
        $('#wGraveyard').empty();
        $('#bGraveyard').empty();
        CHESSBOARD._drawBoard();
        CHESSBOARD._spawnPieces();
    },
    initChessBoard: function (gameId) {
        CHESSBOARD.pieces = [];
        $('#chessBoard').empty();
        $('#board_chat_output').empty();
        $('#wGraveyard').empty();
        $('#bGraveyard').empty();
        $('#board_chat_input').val("");
        $('#wSit').css('display', 'block');
        $('#bSit').css('display', 'block');
        CHESSBOARD.stopTimer();
        CHESSBOARD._drawBoard();
        CHESSBOARD._createPieces();
        CHESSBOARD._spawnPieces();
        CHESSBOARD.gameId = gameId;
        CHESSBOARD.playerA = "";
        CHESSBOARD.playerB = "";
        CHESSBOARD.blackPlayer = '';
        CHESSBOARD.whitePlayer = '';
        CHESSBOARD.currentGameTurn = '';
        CHESSBOARD.gameHistory = [];
        return 0;
    },
    _isCheck: function (pieceId, destinationId) {
        var color = pieceId[0];
        var result = false;
        var previousSqId = CHESSBOARD.pieces[pieceId].sqId;
        var previousDestinationSquareOccupant = CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId)];
        CHESSBOARD.chessBoard[CHESSBOARD._numeric(CHESSBOARD.pieces[pieceId].sqId)] = '';
        CHESSBOARD.pieces[pieceId].sqId = destinationId;
        CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId)] = pieceId;
        if (previousDestinationSquareOccupant) {
            CHESSBOARD.pieces[previousDestinationSquareOccupant].sqId = 'out';
        }
        if (pieceId === 'wking' || pieceId === 'bking') {
            for (var p in CHESSBOARD.pieces) {
                if (CHESSBOARD.pieces[p].type === 'king'
                    && CHESSBOARD.pieces[p].sqId[0] === 's'
                    && p[0] !== pieceId[0]
                    && CHESSBOARD._attack_by_king(p, destinationId)) {
                    result = true;
                } else if (CHESSBOARD.pieces[p].type !== 'king'
                    && CHESSBOARD.pieces[p].sqId[0] === 's'
                    && p[0] !== pieceId[0]
                    && CHESSBOARD._canMove(p, destinationId, true)) {
                    result = p;
                }
            }
        } else {
            if (CHESSBOARD._isCheck(color + 'king', CHESSBOARD.pieces[color + 'king'].sqId)) {
                result = true;
            } else {
                result = false;
            }
        }

        // restore the arrays and return result
        if (previousDestinationSquareOccupant) {
            CHESSBOARD.pieces[previousDestinationSquareOccupant].sqId = destinationId;
        }
        CHESSBOARD.chessBoard[CHESSBOARD._numeric(previousSqId)] = pieceId;
        CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId)] = previousDestinationSquareOccupant;
        CHESSBOARD.pieces[pieceId].sqId = previousSqId;
        return result;
    },
    _markSquare: function (squareId, cssClass, bool) {
        if (bool) {
            try {
                if (squareId === 'ALL') {
                    $('.chessBoardSquare').addClass(cssClass); // JQUERY
                } else {
                    $("#" + squareId).addClass(cssClass); // JQUERY
                }
            } catch (err) {
                //console.log(squareId + ' cannot be marked as ' + cssClass);
                return false;
            }
            if (cssClass != 'check' && CHESSBOARD.canSelect) {
                arr = CHESSBOARD.generateMovePositions(CHESSBOARD.chessBoard[CHESSBOARD._numeric(squareId)]);
                for (x = 0; x < arr.length; x++)
                    CHESSBOARD.hightLightSelected(arr[x]);
            }
            return true;
        } else {
            try {
                if (squareId === 'ALL') {
                    $('.chessBoardSquare').removeClass(cssClass); // JQUERY
                } else {
                    $("#" + squareId).removeClass(cssClass); // JQUERY
                }
            } catch (err) {
                //console.log(squareId + ' cannot be unmarked as ' + cssClass);
                return false;
            }
            CHESSBOARD.clearhightLightSelected();
            return true;
        }
    },
    mouseDownHandler: function (ev) {
        ev.stopPropagation();
        if (CHESSBOARD.checkedWin)
            return false;
        // user is not holding a piece yet and is clicking on one, but the clicked piece is not the right color
        if (ev.target.tagName === 'IMG' && !CHESSBOARD.selectedPiece
            && (ev.target.id.charAt(0) !== CHESSBOARD.currentGameTurn
                || CHESSBOARD.pieces[ev.target.id].sqId === 'wGraveyard' || CHESSBOARD.pieces[ev.target.id].sqId === 'bGraveyard'
                || ev.target.id.charAt(0) === "w" && CHESSBOARD.whitePlayer !== CONTEXT.key
                || ev.target.id.charAt(0) === "b" && CHESSBOARD.blackPlayer !== CONTEXT.key)) {
            return false;
        }
        // user is not holding a piece yet and is clicking on one
        else if (ev.target.tagName === 'IMG' && CHESSBOARD.selectedPiece === '') {
            CHESSBOARD.selectedPiece = $("#" + ev.target.id); // JQUERY
            CHESSBOARD._markSquare(CHESSBOARD.pieces[ev.target.id].sqId, "selected", true);
            return true;
        }
        // user is holding a piece and is selecting another one, of the same color
        else if (ev.target.tagName === 'IMG' && CHESSBOARD.selectedPiece != '' && CHESSBOARD.selectedPiece.attr('id') !== ev.target.id && CHESSBOARD.selectedPiece.attr('id').charAt(0) === ev.target.id.charAt(0)) {
            CHESSBOARD._markSquare(CHESSBOARD.pieces[CHESSBOARD.selectedPiece.attr('id')].sqId, "selected", false);
            CHESSBOARD.selectedPiece = $("#" + ev.target.id); // JQUERY
            CHESSBOARD._markSquare(CHESSBOARD.pieces[ev.target.id].sqId, "selected", true);
            return true;
        }
        // user is holding a piece and is selecting the same piece again
        else if (ev.target.tagName === 'IMG' && CHESSBOARD.selectedPiece != '' && CHESSBOARD.selectedPiece.attr('id') === ev.target.id) {
            CHESSBOARD._markSquare(CHESSBOARD.pieces[CHESSBOARD.selectedPiece.attr('id')].sqId, "selected", false);
            CHESSBOARD.selectedPiece = '';
            return true;
        }
        // user is holding a piece and is clicking on an empty square or a piece of different color
        else if ((ev.target.tagName === 'DIV' || ev.target.tagName === 'IMG' &&
            CHESSBOARD.selectedPiece.attr('id').charAt(0) != ev.target.id.charAt(0) &&
            CHESSBOARD.pieces[ev.target.id].sqId !== '') && CHESSBOARD.selectedPiece) {
            var target = ev.target;
            var pieceId = CHESSBOARD.selectedPiece.attr('id');
            while (target.tagName !== 'DIV') { // if the target was not a DIV, go up in the DOM to find the first DIV
                target = target.parentNode;
            }
            if (!CHESSBOARD._canMove(pieceId, target.id) || CHESSBOARD._isCheck(pieceId, target.id))
                return false;
            CHESSBOARD._markSquare(CHESSBOARD.pieces[CHESSBOARD.selectedPiece.attr('id')].sqId, "selected", false);
            //console.log(pieceId, target.id);
            if (CHESSBOARD.chessBoard[CHESSBOARD._numeric(target.id)] === '' || CHESSBOARD.chessBoard[CHESSBOARD._numeric(target.id)].charAt(0) != pieceId.charAt(0))
                CHESSBOARD.move(pieceId, target.id, false);
            CHESSBOARD.selectedPiece = '';
            return true;
        } else {
            return false;
        }
    },
    move: function (pieceId, destinationId, dontSwitchTurn, notbacktoserver) {
        try {
        var destinationSelector = $('#' + destinationId);
        var pieceSelector = $('#' + pieceId);
        // piece capture
        if (destinationId[0] === 's') {
            var targetPiece1 = '';
            var targetPiece2 = '';
            if (CHESSBOARD._numeric(destinationId) % 10 >= 2 && CHESSBOARD._numeric(destinationId) % 10 <= 7) {
                targetPiece1 = CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId) - 1];
                targetPiece2 = CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId) + 1];
                if (targetPiece1 !== '' && targetPiece2 !== '') {
                    if (targetPiece1[0] !== pieceId[0] && targetPiece2[0] !== pieceId[0]) {
                        CHESSBOARD.move(targetPiece1, CHESSBOARD.pieces[targetPiece1].id[0] + 'Graveyard', true);
                        CHESSBOARD.move(targetPiece2, CHESSBOARD.pieces[targetPiece2].id[0] + 'Graveyard', true);
                        if (CHESSBOARD.pieces[targetPiece1].type === 'king' || CHESSBOARD.pieces[targetPiece2].type === 'king') {
                            CHESSBOARD.checkedWin = true;
                            CHESSBOARD.stopTimer();
                            oukonline.boardWaiting(true, (pieceId[0] === 'w' ? 'White' : 'Black') + ' wins.', true, 'opacity', 0.7);
                        }
                    }
                }
            }
            var targetPiece1 = '';
            var targetPiece2 = '';
            if (Math.floor(CHESSBOARD._numeric(destinationId) / 10) >= 2 && Math.floor(CHESSBOARD._numeric(destinationId) / 10) <= 7) {
                targetPiece1 = CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId) - 10];
                targetPiece2 = CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId) + 10];
                if (targetPiece1 !== '' && targetPiece2 !== '') {
                    if (targetPiece1[0] !== pieceId[0] && targetPiece2[0] !== pieceId[0]) {
                        CHESSBOARD.move(targetPiece1, CHESSBOARD.pieces[targetPiece1].id[0] + 'Graveyard', true);
                        CHESSBOARD.move(targetPiece2, CHESSBOARD.pieces[targetPiece2].id[0] + 'Graveyard', true);
                        if (CHESSBOARD.pieces[targetPiece1].type === 'king' || CHESSBOARD.pieces[targetPiece2].type === 'king') {
                            CHESSBOARD.checkedWin = true;
                            CHESSBOARD.stopTimer();
                            oukonline.boardWaiting(true, (pieceId[0] === 'w' ? 'White' : 'Black') + ' wins.', true, 'opacity', 0.7);
                        }
                    }
                }
            }
        }
        // reflect the move into the chessBoard array
        CHESSBOARD.chessBoard[CHESSBOARD._numeric(CHESSBOARD.pieces[pieceId].sqId)] = '';
        if (destinationId[0] === 's') { // it's a square, not a graveyard
            CHESSBOARD.chessBoard[CHESSBOARD._numeric(destinationId)] = pieceId;
            CHESSBOARD.highLightMoved(CHESSBOARD.pieces[pieceId].sqId, destinationId);
        }
        var marginLeft = destinationSelector.width() * 0.05; // change this if you change the width of the pieces in chessboard.css
        pieceSelector.animate({
                top: "+=" + (destinationSelector.position().top - pieceSelector.position().top) + "px",
                left: "+=" + (destinationSelector.position().left - pieceSelector.position().left + marginLeft) + "px"
            },
            "fast",
            undefined,
            function () {
                pieceSelector.prependTo(destinationSelector);
                pieceSelector.css({
                    top: "0px",
                    left: "0px"
                });
                setTimeout(function () {
                    pieceSelector.css({
                        top: "0px",
                        left: "0px"
                    });
                }, 500);
                setTimeout(function () {
                    pieceSelector.css({
                        top: "0px",
                        left: "0px"
                    });
                }, 1000);
            });
        CHESSBOARD.pieces[pieceId].sqId = destinationId;
        if (!dontSwitchTurn) {
            //CHESSBOARD._commitMoves(CHESSBOARD.gameId); raksa commit move
            if (pieceId[0] === 'w' && CHESSBOARD.whitePlayer === CONTEXT.key && CHESSBOARD.currentGameTurn === 'w'
                || pieceId[0] === 'b' && CHESSBOARD.blackPlayer === CONTEXT.key && CHESSBOARD.currentGameTurn === 'b') {
                var tmp = 'move-' + pieceId + '-' + destinationId;
                if (!notbacktoserver)
                    CHESSBOARD._CommitMoves(tmp);
            }
            CHESSBOARD.killSerounded(destinationId, pieceId[0]);
            CHESSBOARD.currentGameTurn = (pieceId[0] === 'w' ? 'b' : 'w');
            CHESSBOARD.startTimer(CHESSBOARD.currentGameTurn);
            CHESSBOARD._verifyWinner(CHESSBOARD.currentGameTurn);
        }
        } catch (e) {}
    },
    _numeric: function (id) {
        var value = parseInt(CHESSBOARD.numericColumns[id[2]], 10) + parseInt(id[3], 10);
        return value;
    },
    setPlayer: function (parameter, value) {
        if (parameter === 'A') {
            CHESSBOARD.playerA = value;
        } else if (parameter === 'B') {
            CHESSBOARD.playerB = value;
        } else if (parameter === 'w') {
            CHESSBOARD.whitePlayer = value;
            if (value) {
                $('#wSit').css('display', 'none');
                self.socket.emit("getname", {
                    key: value
                }, function (response) {
                    CHESSBOARD.playersnames.whitePlayer = "(w)" + response.name;
                    $('#wTimer').html("(w)" + response.name);
                });
            } else {
                CHESSBOARD.playersnames.whitePlayer = '';
                CHESSBOARD.stopTimer();
                $('#wSit').css('display', 'block');
                $('#wTimer').html("white");
            }
        } else if (parameter === 'b') {
            CHESSBOARD.blackPlayer = value;
            if (value) {
                $('#bSit').css('display', 'none');
                self.socket.emit("getname", {
                    key: value
                }, function (response) {
                    CHESSBOARD.playersnames.blackPlayer = "(b)" + response.name;
                    $('#bTimer').html("(b)" + response.name);
                });
            } else {
                CHESSBOARD.playersnames.blackPlayer = '';
                CHESSBOARD.stopTimer();
                $('#bSit').css('display', 'block');
                $('#bTimer').html("black");
            }
        }
        if (CHESSBOARD.blackPlayer && CHESSBOARD.whitePlayer) {
            if (!CHESSBOARD.currentGameTurn) {
                CHESSBOARD.currentGameTurn = 'w';
                $('#wTimer').addClass('current');
            }
            oukonline.boardWaiting(false, '');
        } else if (CHESSBOARD.whitePlayer === CONTEXT.key || CHESSBOARD.blackPlayer === CONTEXT.key) {
            $('#wSit').css('display', 'none');
            $('#bSit').css('display', 'none');
            oukonline.boardWaiting(true, 'Wait for partner');
        }
        if (CHESSINFO.key === CHESSBOARD.whitePlayer || CHESSINFO.key === CHESSBOARD.blackPlayer)
            $("#cbprivate").show();
        else
            $("#cbprivate").hide();
    },
    _spawnPieces: function () {
        var bGraveyard = $('#bGraveyard');
        var wGraveyard = $('#wGraveyard');
        for (var piece in CHESSBOARD.pieces) {
            var pieceRepresentation = '<img class="piece ' + CHESSBOARD.pieces[piece].classname + '" src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAABCAYAAAAfFcSJAAAABmJLR0QA/wD/AP+gvaeTAAAACXBIWXMAAAsTAAALEwEAmpwYAAAAB3RJTUUH3AseCA0reT+x/AAAAB1pVFh0Q29tbWVudAAAAAAAQ3JlYXRlZCB3aXRoIEdJTVBkLmUHAAAADUlEQVQI12NgYGBgAAAABQABXvMqOgAAAABJRU5ErkJggg==" id="' + CHESSBOARD.pieces[piece].id + '"> </img>';
            if (CHESSBOARD.pieces[piece].id[0] === 'w' && CHESSBOARD.pieces[piece].sqId === '') { // piece is white and captured
                wGraveyard.append(pieceRepresentation);
            } else if (CHESSBOARD.pieces[piece].id[0] === 'b' && CHESSBOARD.pieces[piece].sqId === '') { // piece is black and captured
                bGraveyard.append(pieceRepresentation);
            } else {
                $('#' + CHESSBOARD.pieces[piece].sqId).append(pieceRepresentation);
            }
        }
        $(".piece").bind('vmousedown', function (event) {
            CHESSBOARD.mouseDownHandler(event);
        })
            .on('dragstart', function (event) {
                event.preventDefault();
            });
    },
    startTimer: function (color) {
        CHESSBOARD.stopTimer();
        //raksa timer
        // currentcolor = CHESSINFO.key === CHESSBOARD.blackPlayer ? 'b' : CHESSINFO.key === CHESSBOARD.whitePlayer ? 'w' : '';
        // //console.log(color+" "+CHESSINFO.key+" "+CHESSBOARD.blackPlayer+" "+CHESSBOARD.whitePlayer);
        // if(color !== currentcolor){
        // $('#'+color+'Timer').html('?');
        // return;
        // }
        if (color === 'w') {
            $('#wTimer').addClass('current');
            if (CHESSBOARD.gameTimer > 0) {
                CHESSBOARD.timerInterval = setInterval(function () {
                    if (CHESSBOARD.whiteTimer > 0) {
                        CHESSBOARD.whiteTimer = CHESSBOARD.whiteTimer - 1;
                    }
                    CHESSBOARD.updateTimer(color, CHESSBOARD.whiteTimer);
                }, 1000);
            }
        } else if (color === 'b') {
            $('#bTimer').addClass('current');
            if (CHESSBOARD.gameTimer > 0) {
                CHESSBOARD.timerInterval = setInterval(function () {
                    if (CHESSBOARD.blackTimer > 0) {
                        CHESSBOARD.blackTimer = CHESSBOARD.blackTimer - 1;
                    }
                    CHESSBOARD.updateTimer(color, CHESSBOARD.blackTimer);
                }, 1000);
            }
        }
    },
    stopTimer: function () {
        clearInterval(CHESSBOARD.timerInterval);
        $('.timer').removeClass('current');
    },
    _unnumeric: function (value) {
        var textValue = value + "";
        return 'sq' + CHESSBOARD.unnumericColumns[textValue[0]] + textValue[1];
    },
    updateLastcoutview: function (color) {
        if (CHESSBOARD.gameTimer <= 0) {
            if (color === 'w') {
                $('#wTimer').html((CHESSBOARD.playersnames.whitePlayer.length > 0 ? CHESSBOARD.playersnames.whitePlayer : 'white') + (CHESSBOARD.lastcout[color] > -1 ? ' <span style="color:red;">[</span>' + CHESSBOARD.lastcout[color] + '<span style="color:red;">]</span>' : ''));
            } else {
                $('#bTimer').html((CHESSBOARD.playersnames.blackPlayer.length > 0 ? CHESSBOARD.playersnames.blackPlayer : 'black') + (CHESSBOARD.lastcout[color] > -1 ? ' <span style="color:red;">[</span>' + CHESSBOARD.lastcout[color] + '<span style="color:red;">]</span>' : ''));
            }
        }
    },
    updateTimer: function (color, timer) {
        var minutes = 0;
        var seconds = 0;
        if (timer > 0) {
            minutes = Math.floor(timer / 60);
            seconds = timer - (minutes * 60);
        }
        var string = (minutes < 10 ? "0" : "") + minutes + ":" + (seconds < 10 ? "0" : "") + seconds;
        $('#' + color + 'Timer').html(string + (CHESSBOARD.lastcout[color] > -1 ? ' (' + CHESSBOARD.lastcout[color] + ')' : ''));
        if (timer <= 0) {
            CHESSBOARD.stopTimer();
            if (CHESSBOARD.gameTimer > 0)
                CHESSBOARD.checkedWin = true;
            setTimeout(function () {
                oukonline.boardWaiting(true, (color === 'b' ? 'White' : 'Black') + ' wins', true, 'opacity', 0.7);
            }, 500);
        }
        ;
    },
    updateTimers: function (whiteTimer, blackTimer) {
        CHESSBOARD.whiteTimer = whiteTimer;
        CHESSBOARD.blackTimer = blackTimer;
        CHESSBOARD.updateTimer('w', CHESSBOARD.whiteTimer);
        CHESSBOARD.updateTimer('b', CHESSBOARD.blackTimer);
    },
    _verifyCheck: function (color) {
        try {
        var kings = ['bking', 'wking'];
        CHESSBOARD._markSquare('ALL', 'check', false); // remove all check markers
        kings.forEach(function (king, index) {
            var p = CHESSBOARD._isCheck(king, CHESSBOARD.pieces[king].sqId); // p is the "checker"
            if (p) {
                CHESSBOARD._markSquare(CHESSBOARD.pieces[king].sqId, 'check', true); // mark the king as checked
                CHESSBOARD._markSquare(CHESSBOARD.pieces[p].sqId, 'check', true); // mark the "checker" as ... checked :p
                CHESSBOARD.pieces[king].isCheck = true;
                if (king[0] !== color) {
                    if (!CHESSBOARD._verifyWin(king[0]))
                        CHESSBOARD._verifyLastcount(king[0]);
                }
                return true;
            } else {
                CHESSBOARD._markSquare(CHESSBOARD.pieces[king].sqId, 'check', false);
                CHESSBOARD.pieces[king].isCheck = false;
                if (king[0] !== color) {
                    if (!CHESSBOARD._verifyNowinner(king[0]))
                        CHESSBOARD._verifyLastcount(king[0]);
                }
            }
        });
        } catch (e) {} finally {return false;}
        return false;
    },
    _verifyLastcount: function (color) {
        if (CHESSBOARD.lastcout[color] > 0)
            return false;
        var i = 0;
        for (var x in CHESSBOARD.chessBoard) {
            if (CHESSBOARD.chessBoard[x][0] === color)
                i++;
        }
        if (i == 1) {
            var vscolor = (color === 'w' ? 'b' : 'w');
            var wtmp_pieces = ['wrooka', 'wrookh', 'wbishopc', 'wbishopf', 'wknightb', 'wknightg', 'wqueen'];
            var btmp_pieces = ['brooka', 'brookh', 'bbishopc', 'bbishopf', 'bknightb', 'bknightg', 'bqueen'];
            var p = (color === 'w' ? btmp_pieces : wtmp_pieces);
            if (CHESSBOARD.pieces[p[0]].sqId[0] === 's') {
                if (CHESSBOARD.pieces[p[1]].sqId[0] === 's') {
                    CHESSBOARD.lastcout[color] = 8;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                } else {
                    CHESSBOARD.lastcout[color] = 16;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                }
            } else {
                if (CHESSBOARD.pieces[p[1]].sqId[0] === 's') {
                    CHESSBOARD.lastcout[color] = 16;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                }
            }
            if (CHESSBOARD.pieces[p[2]].sqId[0] === 's') {
                if (CHESSBOARD.pieces[p[3]].sqId[0] === 's') {
                    CHESSBOARD.lastcout[color] = 22;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                } else {
                    if (CHESSBOARD.pieces[p[4]].sqId[0] === 's') {
                        if (CHESSBOARD.pieces[p[5]].sqId[0] === 's') {
                            CHESSBOARD.lastcout[color] = 32;
                            CHESSBOARD.updateLastcoutview(color);
                            return true;
                        }
                    }
                    CHESSBOARD.lastcout[color] = 44;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                }
            } else {
                if (CHESSBOARD.pieces[p[4]].sqId[0] === 's') {
                    if (CHESSBOARD.pieces[p[5]].sqId[0] === 's') {
                        CHESSBOARD.lastcout[color] = 32;
                        CHESSBOARD.updateLastcoutview(color);
                        return true;
                    }
                }
                if (CHESSBOARD.pieces[p[3]].sqId[0] === 's') {
                    CHESSBOARD.lastcout[color] = 44;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                }
            }
            if (CHESSBOARD.pieces[p[4]].sqId[0] === 's') {
                if (CHESSBOARD.pieces[p[5]].sqId[0] === 's') {
                    CHESSBOARD.lastcout[color] = 32;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                } else {
                    CHESSBOARD.lastcout[color] = 64;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                }
            } else {
                if (CHESSBOARD.pieces[p[5]].sqId[0] === 's') {
                    CHESSBOARD.lastcout[color] = 64;
                    CHESSBOARD.updateLastcoutview(color);
                    return true;
                }
            }
            if (CHESSBOARD.pieces[p[6]].sqId[0] === 's') {
                CHESSBOARD.lastcout[color] = 64;
                CHESSBOARD.updateLastcoutview(color);
                return true;
            }
            return true;
        }
        return false;
    },
    _verifyNowinner: function (color) {
        arr = CHESSBOARD.generateMovePositions(color + 'king');
        if (arr.length != 0)
            return false;
        i = 0;
        for (var x in CHESSBOARD.pieces) {
            if (CHESSBOARD.pieces[x].sqId[0] === 's' && CHESSBOARD.pieces[x].id[0] === color) {
                arr = CHESSBOARD.generateMovePositions(x);
                i += arr.length;

            }
        }
        if (i === 0) {
            CHESSBOARD.checkedWin = true;
            CHESSBOARD.stopTimer();
            setTimeout(function () {
                oukonline.boardWaiting(true, (color === 'w' ? 'White' : 'Black') + ' can\'t move. No winner.', true, 'opacity', 0.7);
            }, 500);
            return true;
        }
        return false;
    },
    _verifyWinner: function (color) {
        var bool = true;
        for (var x in CHESSBOARD.pieces) {
            if (CHESSBOARD.pieces[x].sqId[0] === 's' && CHESSBOARD.pieces[x].id[0] === color) {
                for (var i = 1; i <= 8; i++) {
                    for (var j = 1; j <= 8; j++) {
                        var numeric = i * 10 + j;
                        if (CHESSBOARD.chessBoard[numeric][0] !== color) {
                            var sqid = CHESSBOARD._unnumeric(numeric);
                            if (CHESSBOARD._canMove(CHESSBOARD.pieces[x].id, sqid) || CHESSBOARD._canMove(CHESSBOARD.pieces[x].id, sqid, true)) {
                                bool = false;
                            }
                        }
                    }
                }
            }
        }
        if (bool) {
            CHESSBOARD.checkedWin = true;
            CHESSBOARD.stopTimer();
            oukonline.boardWaiting(true, (color === 'w' ? 'Black' : 'White') + ' wins.', true, 'opacity', 0.7);
            return true;
        }
        return false;
    },
    clearMovedHighLight: function () {
        str = 'abcdefgh';
        for (i = 1; i <= 8; i++) {
            //sqh1 7+1
            //sqa8 97+8=105
            for (j = 97; j <= 104; j++) {
                $('#sq' + str.charAt(j - 97) + i).removeClass((i + j) % 2 != 0 ? 'whitehighlight' : 'blackhighlight');
            }
        }
    },
    highLightMoved: function (sq1, sq2) {
        CHESSBOARD.clearMovedHighLight();
        str = 'abcdefgh';
        //sqd5
        i = Number(sq1.charAt(3));
        j = sq1.charCodeAt(2);
        CHESSBOARD.hightLightMoved1('#' + sq1, ((i + j) % 2 != 0 ? true : false));
        i = Number(sq2.charAt(3));
        j = sq2.charCodeAt(2);
        CHESSBOARD.hightLightMoved1('#' + sq2, ((i + j) % 2 != 0 ? true : false));
        CHESSBOARD.playSound('move');
    },
    hightLightMoved1: function (sq, w) {
        if (w)
            $(sq).addClass('whitehighlight');
        else
            $(sq).addClass('blackhighlight');
    },
    clearhightLightSelected: function () {
        str = 'abcdefgh';
        for (i = 1; i <= 8; i++) {
            for (j = 97; j <= 104; j++) {
                $('#sq' + str.charAt(j - 97) + i).removeClass((i + j) % 2 != 0 ? 'whiteselected' : 'blackselected');
            }
        }
    },
    hightLightSelected: function (sq) {
        i = Number(sq.charAt(3));
        j = sq.charCodeAt(2);
        $('#' + sq).addClass((i + j) % 2 != 0 ? 'whiteselected' : 'blackselected');
    },
    playSound: function (command) {
        switch (command) {
            case 'move':
                if (CHESSBOARD.canSound)
                /*
                    $("#beep-moved")[0].play();
                */
                pg.playAudio('beep-moved-mp3');
                break;
            default:
        }
    }
};